@@ -1,14 +1,11 @@
-# Ultralytics YOLO 🚀, AGPL-3.0 license
+# Ultralytics 🚀 AGPL-3.0 License - https://ultralytics.com/license
 
 from itertools import cycle
 
 import cv2
-import matplotlib.pyplot as plt
 import numpy as np
-from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
-from matplotlib.figure import Figure
 
-from ultralytics.solutions.solutions import BaseSolution  # Import a parent class
+from ultralytics.solutions.solutions import BaseSolution, SolutionResults  # Import a parent class
 
 
 class Analytics(BaseSolution):
@@ -32,34 +29,40 @@ class Analytics(BaseSolution):
         clswise_count (Dict[str, int]): Dictionary for class-wise object counts.
         fig (Figure): Matplotlib figure object for the chart.
         ax (Axes): Matplotlib axes object for the chart.
-        canvas (FigureCanvas): Canvas for rendering the chart.
+        canvas (FigureCanvasAgg): Canvas for rendering the chart.
+        lines (dict): Dictionary to store line objects for area charts.
+        color_mapping (Dict[str, str]): Dictionary mapping class labels to colors for consistent visualization.
 
     Methods:
-        process_data: Processes image data and updates the chart.
-        update_graph: Updates the chart with new data points.
+        process: Process image data and update the chart.
+        update_graph: Update the chart with new data points.
 
     Examples:
         >>> analytics = Analytics(analytics_type="line")
         >>> frame = cv2.imread("image.jpg")
-        >>> processed_frame = analytics.process_data(frame, frame_number=1)
-        >>> cv2.imshow("Analytics", processed_frame)
+        >>> results = analytics.process(frame, frame_number=1)
+        >>> cv2.imshow("Analytics", results.plot_im)
     """
 
     def __init__(self, **kwargs):
         """Initialize Analytics class with various chart types for visual data representation."""
         super().__init__(**kwargs)
 
-        self.type = self.CFG["analytics_type"]  # extract type of analytics
+        import matplotlib.pyplot as plt  # scope for faster 'import ultralytics'
+        from matplotlib.backends.backend_agg import FigureCanvasAgg
+        from matplotlib.figure import Figure
+
+        self.type = self.CFG["analytics_type"]  # type of analytics i.e "line", "pie", "bar" or "area" charts.
         self.x_label = "Classes" if self.type in {"bar", "pie"} else "Frame#"
         self.y_label = "Total Counts"
 
         # Predefined data
-        self.bg_color = "#00F344"  # background color of frame
+        self.bg_color = "#F3F3F3"  # background color of frame
         self.fg_color = "#111E68"  # foreground color of frame
         self.title = "Ultralytics Solutions"  # window name
         self.max_points = 45  # maximum points to be drawn on window
         self.fontsize = 25  # text font size for display
-        figsize = (19.2, 10.8)  # Set output image size 1920 * 1080
+        figsize = self.CFG["figsize"]  # set output image size i.e (12.8, 7.2) -> w = 1280, h = 720
         self.color_cycle = cycle(["#DD00BA", "#042AFF", "#FF4447", "#7D24FF", "#BD00FF"])
 
         self.total_counts = 0  # count variable for storing total counts i.e. for line
@@ -69,30 +72,31 @@ class Analytics(BaseSolution):
         if self.type in {"line", "area"}:
             self.lines = {}
             self.fig = Figure(facecolor=self.bg_color, figsize=figsize)
-            self.canvas = FigureCanvas(self.fig)  # Set common axis properties
+            self.canvas = FigureCanvasAgg(self.fig)  # Set common axis properties
             self.ax = self.fig.add_subplot(111, facecolor=self.bg_color)
             if self.type == "line":
                 (self.line,) = self.ax.plot([], [], color="cyan", linewidth=self.line_width)
         elif self.type in {"bar", "pie"}:
             # Initialize bar or pie plot
             self.fig, self.ax = plt.subplots(figsize=figsize, facecolor=self.bg_color)
-            self.canvas = FigureCanvas(self.fig)  # Set common axis properties
+            self.canvas = FigureCanvasAgg(self.fig)  # Set common axis properties
             self.ax.set_facecolor(self.bg_color)
             self.color_mapping = {}
 
             if self.type == "pie":  # Ensure pie chart is circular
                 self.ax.axis("equal")
 
-    def process_data(self, im0, frame_number):
+    def process(self, im0, frame_number):
         """
-        Processes image data and runs object tracking to update analytics charts.
+        Process image data and run object tracking to update analytics charts.
 
         Args:
             im0 (np.ndarray): Input image for processing.
             frame_number (int): Video frame number for plotting the data.
 
         Returns:
-            (np.ndarray): Processed image with updated analytics chart.
+            (SolutionResults): Contains processed image `plot_im`, 'total_tracks' (int, total number of tracked objects)
+                and 'classwise_count' (dict, per-class object count).
 
         Raises:
             ModuleNotFoundError: If an unsupported chart type is specified.
@@ -100,30 +104,31 @@ class Analytics(BaseSolution):
         Examples:
             >>> analytics = Analytics(analytics_type="line")
             >>> frame = np.zeros((480, 640, 3), dtype=np.uint8)
-            >>> processed_frame = analytics.process_data(frame, frame_number=1)
+            >>> results = analytics.process(frame, frame_number=1)
         """
         self.extract_tracks(im0)  # Extract tracks
-
         if self.type == "line":
             for _ in self.boxes:
                 self.total_counts += 1
-            im0 = self.update_graph(frame_number=frame_number)
+            plot_im = self.update_graph(frame_number=frame_number)
             self.total_counts = 0
         elif self.type in {"pie", "bar", "area"}:
             self.clswise_count = {}
-            for box, cls in zip(self.boxes, self.clss):
+            for cls in self.clss:
                 if self.names[int(cls)] in self.clswise_count:
                     self.clswise_count[self.names[int(cls)]] += 1
                 else:
                     self.clswise_count[self.names[int(cls)]] = 1
-            im0 = self.update_graph(frame_number=frame_number, count_dict=self.clswise_count, plot=self.type)
+            plot_im = self.update_graph(frame_number=frame_number, count_dict=self.clswise_count, plot=self.type)
         else:
             raise ModuleNotFoundError(f"{self.type} chart is not supported ❌")
-        return im0
+
+        # return output dictionary with summary for more usage
+        return SolutionResults(plot_im=plot_im, total_tracks=len(self.track_ids), classwise_count=self.clswise_count)
 
     def update_graph(self, frame_number, count_dict=None, plot="line"):
         """
-        Updates the graph with new data for single or multiple classes.
+        Update the graph with new data for single or multiple classes.
 
         Args:
             frame_number (int): The current frame number.
@@ -135,10 +140,10 @@ class Analytics(BaseSolution):
             (np.ndarray): Updated image containing the graph.
 
         Examples:
-            >>> analytics = Analytics()
-            >>> frame_number = 10
-            >>> count_dict = {"person": 5, "car": 3}
-            >>> updated_image = analytics.update_graph(frame_number, count_dict, plot="bar")
+            >>> analytics = Analytics(analytics_type="bar")
+            >>> frame_num = 10
+            >>> results_dict = {"person": 5, "car": 3}
+            >>> updated_image = analytics.update_graph(frame_num, results_dict, plot="bar")
         """
         if count_dict is None:
             # Single line update
@@ -170,7 +175,7 @@ class Analytics(BaseSolution):
                 for key in count_dict.keys():
                     y_data_dict[key] = np.append(y_data_dict[key], float(count_dict[key]))
                     if len(y_data_dict[key]) < max_length:
-                        y_data_dict[key] = np.pad(y_data_dict[key], (0, max_length - len(y_data_dict[key])), "constant")
+                        y_data_dict[key] = np.pad(y_data_dict[key], (0, max_length - len(y_data_dict[key])))
                 if len(x_data) > self.max_points:
                     x_data = x_data[1:]
                     for key in count_dict.keys():
@@ -216,7 +221,7 @@ class Analytics(BaseSolution):
                 self.ax.clear()
 
                 # Create pie chart and create legend labels with percentages
-                wedges, autotexts = self.ax.pie(
+                wedges, _ = self.ax.pie(
                     counts, labels=labels, startangle=start_angle, textprops={"color": self.fg_color}, autopct=None
                 )
                 legend_labels = [f"{label} ({percentage:.1f}%)" for label, percentage in zip(labels, percentages)]
