@@ -4,7 +4,7 @@ Benchmark a YOLO model formats for speed and accuracy.
 
 Usage:
     from ultralytics.utils.benchmarks import ProfileModels, benchmark
-    ProfileModels(['yolo11n.yaml', 'yolov8s.yaml']).profile()
+    ProfileModels(['yolo11n.yaml', 'yolov8s.yaml']).run()
     benchmark(model='yolo11n.pt', imgsz=160)
 
 Format                  | `format=argument`         | Model
@@ -36,13 +36,12 @@ from pathlib import Path
 
 import numpy as np
 import torch.cuda
-import yaml
 
 from ultralytics import YOLO, YOLOWorld
 from ultralytics.cfg import TASK2DATA, TASK2METRIC
 from ultralytics.engine.exporter import export_formats
-from ultralytics.utils import ARM64, ASSETS, LINUX, LOGGER, MACOS, TQDM, WEIGHTS_DIR
-from ultralytics.utils.checks import IS_PYTHON_3_12, check_imgsz, check_requirements, check_yolo, is_rockchip
+from ultralytics.utils import ARM64, ASSETS, LINUX, LOGGER, MACOS, TQDM, WEIGHTS_DIR, YAML
+from ultralytics.utils.checks import IS_PYTHON_3_13, check_imgsz, check_requirements, check_yolo, is_rockchip
 from ultralytics.utils.downloads import safe_download
 from ultralytics.utils.files import file_size
 from ultralytics.utils.torch_utils import get_cpu_info, select_device
@@ -119,13 +118,13 @@ def benchmark(
                     "CoreML and TF.js export only supported on macOS and non-aarch64 Linux"
                 )
             if i in {5}:  # CoreML
-                assert not IS_PYTHON_3_12, "CoreML not supported on Python 3.12"
-            if i in {6, 7, 8}:  # TF SavedModel, TF GraphDef, and TFLite
-                assert not isinstance(model, YOLOWorld), "YOLOWorldv2 TensorFlow exports not supported by onnx2tf yet"
-            if i in {9, 10}:  # TF EdgeTPU and TF.js
+                assert not IS_PYTHON_3_13, "CoreML not supported on Python 3.13"
+            if i in {6, 7, 8, 9, 10}:  # TF SavedModel, TF GraphDef, and TFLite, TF EdgeTPU and TF.js
                 assert not isinstance(model, YOLOWorld), "YOLOWorldv2 TensorFlow exports not supported by onnx2tf yet"
+                # assert not IS_PYTHON_MINIMUM_3_12, "TFLite exports not supported on Python>=3.12 yet"
             if i == 11:  # Paddle
                 assert not isinstance(model, YOLOWorld), "YOLOWorldv2 Paddle exports not supported yet"
+                assert model.task != "obb", "Paddle OBB bug https://github.com/PaddlePaddle/Paddle/issues/72024"
                 assert not is_end2end, "End-to-end models not supported by PaddlePaddle yet"
                 assert LINUX or MACOS, "Windows Paddle exports not supported yet"
             if i == 12:  # MNN
@@ -136,7 +135,7 @@ def benchmark(
                 assert not is_end2end
                 assert not isinstance(model, YOLOWorld), "YOLOWorldv2 IMX exports not supported"
                 assert model.task == "detect", "IMX only supported for detection task"
-                assert "C2f" in model.__str__(), "IMX only supported for YOLOv8"
+                assert "C2f" in model.__str__(), "IMX only supported for YOLOv8"  # TODO: enable for YOLO11
             if i == 15:  # RKNN
                 assert not isinstance(model, YOLOWorld), "YOLOWorldv2 RKNN exports not supported yet"
                 assert not is_end2end, "End-to-end models not supported by RKNN yet"
@@ -177,7 +176,7 @@ def benchmark(
         except Exception as e:
             if verbose:
                 assert type(e) is AssertionError, f"Benchmark failure for {name}: {e}"
-            LOGGER.warning(f"ERROR ❌️ Benchmark failure for {name}: {e}")
+            LOGGER.error(f"Benchmark failure for {name}: {e}")
             y.append([name, emoji, round(file_size(filename), 1), None, None, None])  # mAP, t_inference
 
     # Print results
@@ -273,7 +272,7 @@ class RF100Benchmark:
                     if not Path(proj_version).exists():
                         self.rf.workspace(workspace).project(project).version(version).download("yolov8")
                     else:
-                        print("Dataset already downloaded.")
+                        LOGGER.info("Dataset already downloaded.")
                     self.ds_cfg_list.append(Path.cwd() / proj_version / "data.yaml")
                 except Exception:
                     continue
@@ -283,12 +282,10 @@ class RF100Benchmark:
     @staticmethod
     def fix_yaml(path):
         """Fix the train and validation paths in a given YAML file."""
-        with open(path, encoding="utf-8") as file:
-            yaml_data = yaml.safe_load(file)
+        yaml_data = YAML.load(path)
         yaml_data["train"] = "train/images"
         yaml_data["val"] = "valid/images"
-        with open(path, "w", encoding="utf-8") as file:
-            yaml.safe_dump(yaml_data, file)
+        YAML.dump(yaml_data, path)
 
     def evaluate(self, yaml_path, val_log_file, eval_log_file, list_ind):
         """
@@ -309,8 +306,7 @@ class RF100Benchmark:
             >>> benchmark.evaluate("path/to/data.yaml", "path/to/val_log.txt", "path/to/eval_log.txt", 0)
         """
         skip_symbols = ["🚀", "⚠️", "💡", "❌"]
-        with open(yaml_path, encoding="utf-8") as stream:
-            class_names = yaml.safe_load(stream)["names"]
+        class_names = YAML.load(yaml_path)["names"]
         with open(val_log_file, encoding="utf-8") as f:
             lines = f.readlines()
             eval_lines = []
@@ -335,12 +331,12 @@ class RF100Benchmark:
                 )
         map_val = 0.0
         if len(eval_lines) > 1:
-            print("There's more dicts")
+            LOGGER.info("Multiple dicts found")
             for lst in eval_lines:
                 if lst["class"] == "all":
                     map_val = lst["map50"]
         else:
-            print("There's only one dict res")
+            LOGGER.info("Single dict found")
             map_val = [res["map50"] for res in eval_lines][0]
 
         with open(eval_log_file, "a", encoding="utf-8") as f:
@@ -378,7 +374,7 @@ class ProfileModels:
         Profile models and print results
         >>> from ultralytics.utils.benchmarks import ProfileModels
         >>> profiler = ProfileModels(["yolo11n.yaml", "yolov8s.yaml"], imgsz=640)
-        >>> profiler.profile()
+        >>> profiler.run()
     """
 
     def __init__(
@@ -403,7 +399,7 @@ class ProfileModels:
             imgsz (int): Size of the image used during profiling.
             half (bool): Flag to indicate whether to use FP16 half-precision for TensorRT profiling.
             trt (bool): Flag to indicate whether to profile using TensorRT.
-            device (torch.device | None): Device used for profiling. If None, it is determined automatically.
+            device (torch.device | str | None): Device used for profiling. If None, it is determined automatically.
 
         Notes:
             FP16 'half' argument option removed for ONNX as slower on CPU than FP32.
@@ -412,7 +408,7 @@ class ProfileModels:
             Initialize and profile models
             >>> from ultralytics.utils.benchmarks import ProfileModels
             >>> profiler = ProfileModels(["yolo11n.yaml", "yolov8s.yaml"], imgsz=640)
-            >>> profiler.profile()
+            >>> profiler.run()
         """
         self.paths = paths
         self.num_timed_runs = num_timed_runs
@@ -421,9 +417,9 @@ class ProfileModels:
         self.imgsz = imgsz
         self.half = half
         self.trt = trt  # run TensorRT profiling
-        self.device = device or torch.device(0 if torch.cuda.is_available() else "cpu")
+        self.device = device if isinstance(device, torch.device) else select_device(device)
 
-    def profile(self):
+    def run(self):
         """
         Profile YOLO models for speed and accuracy across various formats including ONNX and TensorRT.
 
@@ -434,12 +430,12 @@ class ProfileModels:
             Profile models and print results
             >>> from ultralytics.utils.benchmarks import ProfileModels
             >>> profiler = ProfileModels(["yolo11n.yaml", "yolov8s.yaml"])
-            >>> results = profiler.profile()
+            >>> results = profiler.run()
         """
         files = self.get_files()
 
         if not files:
-            print("No matching *.pt or *.onnx files found.")
+            LOGGER.warning("No matching *.pt or *.onnx files found.")
             return
 
         table_rows = []
@@ -496,7 +492,7 @@ class ProfileModels:
             else:
                 files.extend(glob.glob(str(path)))
 
-        print(f"Profiling: {sorted(files)}")
+        LOGGER.info(f"Profiling: {sorted(files)}")
         return [Path(file) for file in sorted(files)]
 
     @staticmethod
@@ -693,7 +689,7 @@ class ProfileModels:
         header = "|" + "|".join(f" {h} " for h in headers) + "|"
         separator = "|" + "|".join("-" * (len(h) + 2) for h in headers) + "|"
 
-        print(f"\n\n{header}")
-        print(separator)
+        LOGGER.info(f"\n\n{header}")
+        LOGGER.info(separator)
         for row in table_rows:
-            print(row)
+            LOGGER.info(row)
