@@ -14,10 +14,10 @@ except (ImportError, AssertionError):
 
 def on_fit_epoch_end(trainer):
     """
-    Sends training metrics to Ray Tune at end of each epoch.
+    Reports training metrics to Ray Tune at epoch end when a Ray session is active.
 
-    This function checks if a Ray Tune session is active and reports the current training metrics along with the
-    epoch number to Ray Tune's session.
+    Captures metrics from the trainer object and sends them to Ray Tune with the current epoch number,
+    enabling hyperparameter tuning optimization. Only executes when within an active Ray Tune session.
 
     Args:
         trainer (ultralytics.engine.trainer.BaseTrainer): The Ultralytics trainer object containing metrics and epochs.
@@ -25,6 +25,9 @@ def on_fit_epoch_end(trainer):
     Examples:
         >>> # Called automatically by the Ultralytics training loop
         >>> on_fit_epoch_end(trainer)
+
+    References:
+        Ray Tune docs: https://docs.ray.io/en/latest/tune/index.html
     """
     if ray.train._internal.session.get_session():  # check if Ray Tune session is active
         metrics = trainer.metrics
